from flask_mailjet import Mailjet


def test_normalize_single_recipient():
    mj = Mailjet()
    result = mj._normalize_recipients("test@example.com")
    assert result == [{"Email": "test@example.com"}]


def test_normalize_multiple_recipients():
    mj = Mailjet()
    result = mj._normalize_recipients(["a@example.com", "b@example.com"])
    assert result == [
        {"Email": "a@example.com"},
        {"Email": "b@example.com"},
    ]



import os
import pytest
from flask import Flask
from flask_mailjet.loader import render_email_template


@pytest.fixture
def app():
    # Absolute path to src/flask_mailjet/templates
    root = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "src", "flask_mailjet", "templates"))

    app = Flask(
        __name__,
        template_folder=root
    )
    with app.app_context():
        yield app


def test_template_rendering(app):
    html = render_email_template("base.html", subject="Hello")
    assert "<title>Hello</title>" in html


def test_template_rendering_context(app):
    html = render_email_template("base.html", subject="Hi", user="Tim")
    assert "Tim" in html or "Hi" in html


def test_template_missing(app):
    with pytest.raises(Exception):
        render_email_template("does_not_exist.html")



import pytest
from flask import Flask
from flask_mailjet import Mailjet


@pytest.fixture
def app():
    app = Flask(__name__)
    app.config["MAILJET_API_KEY"] = "x"
    app.config["MAILJET_API_SECRET"] = "y"
    with app.app_context():
        yield app


def test_get_sender_explicit_arguments(app):
    mj = Mailjet(app)
    sender = mj._get_sender(name="Tim", email="tim@example.com")

    assert sender == {
        "Email": "tim@example.com",
        "Name": "Tim",
    }


def test_get_sender_uses_config(app):
    app.config["MAILJET_SENDER_NAME"] = "Config Sender"
    app.config["MAILJET_SENDER_EMAIL"] = "cfg@example.com"

    mj = Mailjet(app)
    sender = mj._get_sender()

    assert sender["Email"] == "cfg@example.com"
    assert sender["Name"] == "Config Sender"


def test_get_sender_fallback_defaults(app):
    # No config specified
    mj = Mailjet(app)
    sender = mj._get_sender()

    assert sender["Email"] == "do-not-reply@change-me.org"
    assert sender["Name"] == "Flask-Mailjet Sender"



from unittest.mock import MagicMock
from flask import Flask
from flask_mailjet import Mailjet


def make_app():
    app = Flask(__name__)
    app.config["MAILJET_API_KEY"] = "x"
    app.config["MAILJET_API_SECRET"] = "y"
    return app


class DummySend:
    """Replacement for client.send so we can mock create()."""
    def __init__(self):
        self.create = MagicMock(return_value={"Status": "success"})


def test_send_email_payload(monkeypatch):
    app = make_app()
    mj = Mailjet(app)

    dummy_send = DummySend()
    # Replace the entire send object
    mj.client.send = dummy_send

    with app.app_context():
        mj.send_email(
            recipients="a@example.com",
            subject="Hello",
            html="<b>Hi</b>",
        )

    # Now this exists because we replaced send entirely
    payload = dummy_send.create.call_args.kwargs["data"]
    msg = payload["Messages"][0]

    assert msg["To"] == [{"Email": "a@example.com"}]
    assert msg["Subject"] == "Hello"
    assert msg["HTMLPart"] == "<b>Hi</b>"
    assert msg["From"]["Email"] == "do-not-reply@change-me.org"


def test_send_email_with_pdf_payload(monkeypatch):
    app = make_app()
    mj = Mailjet(app)

    dummy_send = DummySend()
    mj.client.send = dummy_send

    with app.app_context():
        mj.send_email_with_pdf(
            recipients="a@example.com",
            subject="PDF",
            html="<p>Hello</p>",
            filename="test.pdf",
            file_bytes=b"123456",
        )

    payload = dummy_send.create.call_args.kwargs["data"]
    msg = payload["Messages"][0]
    att = msg["Attachments"][0]

    assert att["Filename"] == "test.pdf"
    assert att["ContentType"] == "application/pdf"
    assert att["Base64Content"] == "MTIzNDU2"



from __future__ import annotations
import base64
from typing import List, Dict, Any, Union
from mailjet_rest import Client
from flask import Flask, current_app


class Mailjet:
    """Flask extension providing Mailjet email sending support."""

    def __init__(self, app: Flask | None = None) -> None:
        self.client: Client | None = None
        if app:
            self.init_app(app)

    def init_app(self, app: Flask) -> None:
        """Bind Mailjet client to the Flask app."""
        api_key = app.config.get("MAILJET_API_KEY")
        api_secret = app.config.get("MAILJET_API_SECRET")

        if not api_key or not api_secret:
            raise RuntimeError(
                "Configuration values MAILJET_API_KEY and MAILJET_API_SECRET are required."
            )

        self.client = Client(
            auth=(api_key, api_secret),
            version="v3.1"
        )

        app.extensions["mailjet"] = self

    # ------------------------------------------------------------------
    # Email Sending API
    # ------------------------------------------------------------------
    def _normalize_recipients(self, recipients: Union[str, List[str]]) -> List[Dict[str, str]]:
        if isinstance(recipients, str):
            return [{"Email": recipients}]
        return [{"Email": r} for r in recipients]

    def _get_sender(self, name: str | None = None, email: str | None = None) -> dict:
        """
        Determine sender name and email from config or provided overrides.

        Priority:
        1. Explicit arguments (name/email)
        2. Flask config MAILJET_SENDER_NAME / MAILJET_SENDER_EMAIL
        3. Final fallback: "Flask-Mailjet Sender" + do-not-reply@change-me.org
        """

        cfg = current_app.config

        sender_name = (
                name
                or cfg.get("MAILJET_SENDER_NAME")
                or "Flask-Mailjet Sender"
        )

        sender_email = (
                email
                or cfg.get("MAILJET_SENDER_EMAIL")
                or "do-not-reply@change-me.org"
        )

        return {
            "Email": sender_email,
            "Name": sender_name,
        }

    def send_email(
        self,
        recipients: Union[str, List[str]],
        subject: str,
        html: str,
        sender: Dict[str, str] | None = None,
    ) -> Any:
        """Send a basic HTML email."""
        if self.client is None:
            raise RuntimeError("Mailjet extension not initialized.")

        if sender is None:
            sender = self._get_sender()

        payload = {
            "Messages": [
                {
                    "From": sender,
                    "To": self._normalize_recipients(recipients),
                    "Subject": subject,
                    "HTMLPart": html,
                }
            ]
        }

        return self.client.send.create(data=payload)

    def send_email_with_pdf(
        self,
        recipients: Union[str, List[str]],
        subject: str,
        html: str,
        filename: str,
        file_bytes: bytes,
        content_type: str = "application/pdf",
        sender: Dict[str, str] | None = None,
    ) -> Any:
        """Send an email with a file attachment."""
        if self.client is None:
            raise RuntimeError("Mailjet extension not initialized.")

        encoded = base64.b64encode(file_bytes).decode("utf-8")

        if sender is None:
            sender = self._get_sender()

        payload = {
            "Messages": [
                {
                    "From": sender,
                    "To": self._normalize_recipients(recipients),
                    "Subject": subject,
                    "HTMLPart": html,
                    "Attachments": [
                        {
                            "ContentType": content_type,
                            "Filename": filename,
                            "Base64Content": encoded,
                        }
                    ],
                }
            ]
        }

        return self.client.send.create(data=payload)


import pytest
from flask import Flask
from flask_mailjet import Mailjet


def test_extension_init():
    app = Flask(__name__)
    app.config["MAILJET_API_KEY"] = "test"
    app.config["MAILJET_API_SECRET"] = "test"

    mj = Mailjet(app)
    assert "mailjet" in app.extensions
    assert mj.client is not None


def test_extension_init_missing_config():
    app = Flask(__name__)
    # No config set -> must raise
    with pytest.raises(RuntimeError):
        Mailjet(app)
